/**
 Copyright (c) 2004-2010 Honda Research Institute Japan, Co., Ltd.

 \file CMICARY_Matrix.h
 \brief \~English Header file for matrix
 \brief \~Japanese s񉉎Zwb_t@C(GDHSS program)
 \author Aki Yamamoto
 \date 22 Feb 2011
 \version $Id: CMICARY_Matrix.h $
*/
// ---------------------------------------------------------------------
#ifndef	CMICARY_Matrix_h_defined
#define	CMICARY_Matrix_h_defined
// ---------------------------------------------------------------------
#include <boost/numeric/ublas/matrix.hpp>
#include <complex>
#include <iostream>
#include <boost/numeric/ublas/io.hpp>      // Matrix Xg[o͗pwb_
#include <stdio.h>

#include "ArrayInit.h"
#include "MICARY_struct.h"
#include "MICARY_GHDSS_struct.h"

//#include "GHDSS_MathMatDouble.h"
// ---------------------------------------------------------------------
using namespace std;
using namespace boost;
using namespace boost::numeric;
using namespace boost::numeric::ublas;
// ---------------------------------------------------------------------
typedef ublas::matrix<complex<double> > cdMat;


// ---------------------------------------------------------------------
#define __MAX_FLOAT_VALUE 3.0428E+038
#define __MIN_FLOAT_VALUE 1.1754E-038
#define __MAX_DOUBLE_VALUE 1.79769313E+308
#define __MIN_DOUBLE_VALUE 2.22507385E-308

#define __FALSE__ 0
#define __TRUE__ 1
// ---------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

// ----------- Add 2011.01.19 -----------------
typedef struct 
{
    /// real part
    double re;
    /// imaginary part
    double im;
} DA_Complex;
/**
 Complex array struct
 \author	Natsu Tanaka
 \date	20 Jul 2005
*/
typedef struct{
	DA_Complex *	da_cmpValue; ///<\~Japanese HA_Complexz
	HA_Num	ha_numValue; ///<\~Japanese z̃TCY
// 2012/01/13 added
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num ha_numReserved; ///<\~Japanese mۗvf
#endif
} MICARY_dComplexArray;

/**
 Spectrum struct (Double Complex)
 \author	Sachiko Suzuki
 \date 01 Jul 2008
*/
typedef struct{
	MICARY_dComplexArray	micary_cmpaSpec; ///<\~Japanese XyNg(Complex) 
	HA_Num	ha_numCh; ///<\~Japanese `l
	HA_Integer	ha_iLowerFreq; ///<\~Japanese g[Line]
	HA_Integer	ha_iUpperFreq; ///<\~Japanese g[Line]
// 2012/01/13 added
#ifdef __MICARY_def_USE_RESERVED_ARRAY
	HA_Num ha_numReservedChSize; ///<\~Japanese mۗvf (ch)
	HA_Num ha_numReservedFreqSize; ///<\~Japanese mۗvf (freq)
#endif
} MICARY_dComplexSpecArray;


// ----------- Add 2011.01.19 -----------------
/**
 \brief \~English It allocates complex array
 \brief \~Japanese HA_Complexz̗̈mۂ
 \param array \~English It's ComplexArray
 \~Japanese HA_Complexz܂ލ\
 \param iArraySize \~English It's the size of array
 \~Japanese mۂvf
*/
void dComplexArrayAlloc(MICARY_dComplexArray * array, int iArraySize);

// 2012/01/13 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates complex array
 \brief \~Japanese HA_Complexz̗̈mۂ
 \param array \English It's complexArray
 \~Japanese HA_Complexz܂ލ\
 \param iArraySize \~English It's the size of array in use
 \~Japanese gpvf
 \param iReservedSize \~English It's the size of array
 \~Japanese mۂvf
*/
void dComplexArrayAlloc_r(MICARY_dComplexArray * array, int iArraySize, int iReservedSize);
#endif

/**
 \brief \~English It frees complex array
 \brief \~Japanese HA_Complexz 
 \param array \~English It's ComplexArray
 \~Japanese HA_Complexz܂ލ\
*/
void dComplexArrayFree(MICARY_dComplexArray * array);

// 2012/01/13 added
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees complex array
 \brief \~Japanese HA_Complexz̎gpTCYNA 
 \param array \~English ComplexArray
 \~Japanese HA_Complexz܂ލ\
*/
void dComplexArrayClear(MICARY_dComplexArray * array);
#endif

	
/**
 \brief \~English It initializes complex array
 \brief \~Japanese Complexz̏(0)
 \param array \~English It's complex array
 \~Japanese Complexz܂ލ\
*/

void dComplexArrayInit(MICARY_dComplexArray * array) ;

// 2008/07/01 added by S.Suzuki
/**
 \brief \~English It allocates complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̗̈mۂ
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
 \param numCh \~English It's the number of channel.
 \~Japanese `l
 \param iLowerIndex \~English It's the lower frequency index.
 \~Japanese gCfbNX
 \param iUpperIndex \~English It's the upper frequency index.
 \~Japanese gCfbNX
*/
void AllocdComplexSpecArray(MICARY_dComplexSpecArray *pcsSpec,HA_Num numCh, HA_Integer iLowerIndex, HA_Integer iUpperIndex);

// 2012/01/13 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̗̈mۂ
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
 \param numCh \~English It's the number of channel.
 \~Japanese `l
 \param iLowerIndex \~English It's the lower frequency index.
 \~Japanese gCfbNX
 \param iUpperIndex \~English It's the upper frequency index.
 \~Japanese gCfbNX
 \param iReservedChSize \~English It's the size of array
 \~Japanese z̊mی(ch)
 \param iReservedFreqSize \~English It's the size of array
 \~Japanese z̊mی(Freq)
*/
void AllocdComplexSpecArray_r(MICARY_dComplexSpecArray *pcsSpec,
						  HA_Num numCh,
						  HA_Integer iLowerIndex,
						  HA_Integer iUpperIndex,
						  int iReservedChSize,
						  int iReservedFreqSize);
#endif

/**
 \brief \~English It frees complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̗̈J
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
*/
void FreedComplexSpecArray(MICARY_dComplexSpecArray *pcsSpec);

// 2012/01/13 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̎gpTCYNA
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
*/
void CleardComplexSpecArray(MICARY_dComplexSpecArray *pcsSpec);
#endif


// ****************** boost/numeric/ublas/ gpȂ *******************
// -----  double  ------

/**
 \brief \~English It products complex matrix.(double)
 \brief \~Japanese fs̊|Z(double)
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g2 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r2 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
*/
int MICARY_Product_CompMat(MICARY_dComplexArray *pcaOutMat, MICARY_dComplexArray caInMat1,int g1,int r1,
									MICARY_dComplexArray caInMat2,int g2,int r2);

/**
 \brief \~English It devids complex matrix.
 \brief \~Japanese fs̊Z
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g2 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r2 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
*/
int MICARY_Divide_CompMat(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1,int g1,int r1,
									MICARY_dComplexArray caInMat2,int g2,int r2);

/**
 \brief \~English It adds complex matrix.
 \brief \~Japanese fŝZ
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g2 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r2 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
*/
int MICARY_Sum_CompMat(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat1,int g1,int r1,
							 MICARY_dComplexArray caInMat2,int g2,int r2);

/**
 \brief \~English It substracts complex matrix.
 \brief \~Japanese fŝЂZ
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g2 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r2 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
*/
int MICARY_Sub_CompMat(MICARY_dComplexArray *pcaOutMat, MICARY_dComplexArray caInMat1,int g1,int r1,
							 MICARY_dComplexArray caInMat2,int g2,int r2);


/**
 \brief \~English It translates conjugate complex matrix.
 \brief \~Japanese ]uf
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int MICARY_ConjTrans_CompMat(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat,int g,int r);

/**
 \brief \~English It makes diagonal complex matrix.
 \brief \~Japanese Ίps(s̏ꍇ)
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the number of row for input matrix.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int MICARY_Diag_CompMat(MICARY_dComplexArray *pcaOutMat, MICARY_dComplexArray caInMat, int g , int r);

/**
 \brief \~English It conjugates complex matrix.
 \brief \~Japanese 𕡑f
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
*/
int MICARY_Conj_CompMat(MICARY_dComplexArray *pcaOutMat, MICARY_dComplexArray caInMat);

/**
 \brief \~English It computes a dot product of a conjugated vector with another vector(0:false 1:true).
 \brief \~Japanese dot|Z(0:G[ 1:I)
 \param pcaOut \~English It's the complex  vector.
 \~Japanese o͕fxNg
 \param caIn1 \~English It's the input vector1.
 \~Japanese ͕fxNg1
 \param caIn2 \~English It's the input vector2.
 \~Japanese ͕fxNg2
*/
int MICARY_Dot_CompMat(MICARY_dComplexArray *pcaOut, MICARY_dComplexArray caIn1, MICARY_dComplexArray caIn2);

/**
 \brief \~English It makes unit matrix.
 \brief \~Japanese Pʍs̍쐬
 \param pcaOutMat \~English It's the complex matrix struct
 \~Japanese o͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int MICARY_Eye_CompMat(MICARY_dComplexArray *pcaOutMat,int g,int r);


/**
 \brief \~English It multiplicates complex matrix.
 \brief \~Japanese fs̒萔{
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex array struct.
 \~Japanese ͕fs
 \param a \~English It's the coeficient value.
 \~Japanese Wl
*/
int MICARY_Multipl_CompMat(MICARY_dComplexArray *pcaOutMat ,MICARY_dComplexArray caInMat, double a);

/*
 \brief \~English It devids complex matrix.
 \brief \~Japanese fs̕
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs
*/
int MICARY_Sqrt_CompMat(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1) ;


/**
 \brief \~English It calculates abs complex matrix.(double)
 \brief \~Japanese s̐ΒľvZ(double)
 \param pcaOutMat \~English It's the complex  matrix.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex  matrix.
 \~Japanese ͕fs
*/
int MICARY_Abs_CompMat(MICARY_dComplexArray *pcaOutMat, MICARY_dComplexArray caInMat) ;


// -----  float  ------

/**
 \brief \~English It substracts complex matrix.
 \brief \~Japanese fŝЂZ
 \param pcaOutMat \~English It's the complex array struct.(float)
 \~Japanese o͕fs(float)
 \param caInMat1 \~English It's the complex array struct.(float)
 \~Japanese ͕fs1(float)
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.(float)
 \~Japanese ͕fs2(float)
 \param g2 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r2 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
*/
int MICARY_Sub_fCompMat(MICARY_ComplexArray *pcaOutMat,MICARY_ComplexArray caInMat1,int g1,int r1,
										MICARY_ComplexArray caInMat2,int g2,int r2);

/**
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.(double)
 \~Japanese o͕fs(double)
 \param caInMat1 \~English It's the number of row for input matrix1.(float)
 \~Japanese ͕fs1(float)
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.(double)
 \~Japanese ͕fs2(double)
 \param g2 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r2 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
*/
int MICARY_Product_fdCompMat(MICARY_dComplexArray *pcaOutMat,MICARY_ComplexArray caInMat1,int g1,int r1,
									MICARY_dComplexArray caInMat2,int g2,int r2);

/**
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.(double)
 \~Japanese o͕fs(double)
 \param caInMat1 \~English It's the number of row for input matrix1.(double)
 \~Japanese ͕fs1(double)
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.(float)
 \~Japanese ͕fs2(float)
 \param g2 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r2 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
*/
int MICARY_Product_dfCompMat(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1,int g1,int r1,
									MICARY_ComplexArray caInMat2,int g2,int r2);


//int MICARY_Product_ddfCompMat(MICARY_dComplexArray *pcaOutMat,MICARY_dComplexArray caInMat1,int g1,int r1,
//									MICARY_ComplexArray caInMat2,int g2,int r2);

/**
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.(float)
 \~Japanese o͕fs(float)
 \param caInMat1 \~English It's the number of row for input matrix1.(float)
 \~Japanese ͕fs1(float)
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.(float)
 \~Japanese ͕fs2(float)
 \param g2 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r2 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
*/
int MICARY_Product_fCompMat(MICARY_ComplexArray *pcaOutMat,MICARY_ComplexArray caInMat1,int g1,int r1,
									MICARY_ComplexArray caInMat2,int g2,int r2);


/**
 \brief \~English It translates conjugate complex matrix.
 \brief \~Japanese ]uf
 \param pcaOutMat \~English It's the complex matrix struct.(double)
 \~Japanese o͕fs(double)
 \param caInMat \~English It's the complex matrix struct.(float)
 \~Japanese ͕fs(float)
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int MICARY_ConjTrans_dfCompMat(MICARY_dComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat,int g,int r);


/**
 \brief \~English It translates conjugate complex matrix.
 \brief \~Japanese ]uf
 \param pcaOutMat \~English It's the complex matrix struct.(float)
 \~Japanese o͕fs(float)
 \param caInMat \~English It's the complex matrix struct.(float)
 \~Japanese ͕fs(float)
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int MICARY_ConjTrans_fCompMat(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat,int g,int r);

/**
 \brief \~English It makes diagonal complex matrix.
 \brief \~Japanese Ίps(s̏ꍇ)
 \param pcaOutMat \~English It's the complex matrix struct.(float)
 \~Japanese o͕fs(float)
 \param caInMat \~English It's the number of row for input matrix.(float)
 \~Japanese ͕fs(float)
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int MICARY_Diag_fCompMat(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat, int g , int r);


/**
 \brief \~English It multiplicates complex matrix.
 \brief \~Japanese fs̒萔{
 \param pcaOutMat \~English It's the complex array struct.(float)
 \~Japanese o͕fs(float)
 \param caInMat \~English It's the complex array struct.(float)
 \~Japanese ͕fs(float)
 \param a \~English It's the coeficient value.
 \~Japanese Wl
*/
int MICARY_Multipl_fCompMat(MICARY_ComplexArray *pcaOutMat, MICARY_ComplexArray caInMat, float a);

/**
 \brief \~English It conjugates complex matrix.
 \brief \~Japanese 𕡑f
 \param pcaOutMat \~English It's the complex matrix struct.(float)
 \~Japanese o͕fs(float)
 \param caInMat \~English It's the complex matrix struct.(float)
 \~Japanese ͕fs(float)
*/
int MICARY_Conj_fCompMat(MICARY_ComplexArray *pcaOutMat, MICARY_ComplexArray caInMat);

/**
 \brief \~English It makes unit matrix.
 \brief \~Japanese Pʍs̍쐬
 \param pcaOutMat \~English It's the complex matrix struct
 \~Japanese o͕fs(float)
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int MICARY_Eye_fCompMat(MICARY_ComplexArray *pcaOutMat,int g,int r);

/**
 \brief \~English It transrates complex matrix.
 \brief \~Japanese f̓]u
 \param pcaOutMat \~English It's the complex matrix struct.(float)
 \~Japanese o͕fs(float)
 \param caInMat \~English It's the complex matrix struct.(float)
 \~Japanese ͕fs(float)
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int MICARY_Trans_fCompMat(MICARY_ComplexArray *pcaOutMat, MICARY_ComplexArray caInMat,int g,int r);

/**
 \brief \~English It calculates abs complex matrix.
 \brief \~Japanese s̐ΒľvZ
 \param pcaOutMat \~English It's the complex  matrix.(float)
 \~Japanese o͕fs(float)
 \param caInMat \~English It's the complex  matrix.(float)
 \~Japanese ͕fs(float)
*/
int MICARY_Abs_dfCompMat(MICARY_dComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat); 

/**
 \brief \~English It calculates max value.
 \brief \~Japanese s(real part)̍őľvZ
 \param caInMat \~English It's the complex  matrix.
 \~Japanese ͕fs
*/
double MICARY_Max_CompMat(MICARY_dComplexArray caInMat);



//   ******************* boost/numeric/ublas/ gp *******************
// 2011.02.16_ł͖gp

bool _InitCompMat(cdMat *pcaOutMat);

bool _DiagCompMat(cdMat *pcaOutMat, cdMat caInMat);

bool _EyeCompMat(cdMat *pcaOutMat);

bool _AbsCompMat(cdMat *pcaOutMat ,cdMat caInMat);


double _MaxCompMat(cdMat caInMat);

bool _SqrtCompMat(cdMat *pcaOutMat ,cdMat caInMat);

bool _MultiplCompMat(cdMat *pcaOutMat ,cdMat caInMat, double a);

//--- uBlas::matrix^ւ̕ϊ ---
bool _Convert_ComplexSpec(cdMat *pcaOutMat, MICARY_ComplexArray caIn, int Nrow, int Ncol, int iFreqN);

bool _Convert_Complex_InputSpec(cdMat *pcaOutMat ,MICARY_ComplexArray caIn, int Nfreq, int iFreq);

bool _Convert_dComplexSpec(cdMat *pcaOutMat ,MICARY_dComplexArray caIn,   int Nrow, int Nmic, int iFreqN);
// ---

//--- uBlas::matrix^̕ϊ
bool _Update_ComplexSpec(MICARY_ComplexArray *caOut, cdMat pcaInMat, int Nrow, int Ncol, int iFreq);

bool _Update_dComplexSpec(MICARY_dComplexArray *caOut, cdMat pcaInMat, int Nrow, int Ncol, int iFreq);

//bool _Update_Complex_InputSpec(MICARY_ComplexArray *caOut, cdMat pcaInMat,   int Nfreq, int iFreq);
//---
//   ********************************************************************

#ifdef __cplusplus
}
#endif
// -------------------------------------------------------------------------------------
#endif // CMICARY_Matrix_h_defined
